@@ -1,12 +1,12 @@
 
 #include <test.hpp>
 
-#include "test/t/double/testcase.pb.h"
+#include "t/double/double_testcase.pb.h"
 
 TEST_CASE("write double field and check with libprotobuf") {
 
     std::string buffer;
-    protozero::pbf_writer pw(buffer);
+    protozero::pbf_writer pw{buffer};
 
     TestDouble::Test msg;
 
